// --------------------------------------------------------------------------
// File: examples/src/InputDataReader.java
// Version 10.1  
// --------------------------------------------------------------------------
//  Copyright (C) 2001-2006 by ILOG.
//  All Rights Reserved.
//  Permission is expressly granted to use this example in the
//  course of developing applications that use ILOG products.
// --------------------------------------------------------------------------
//
// This is a helper class used by several examples to read input data files
// containing arrays in the format [x1, x2, ..., x3].  Up to two-dimensional
// arrays are supported.
//

import java.io.*;
import java.util.*;

public class InputDataReader {
   public static class InputDataReaderException extends Exception {
      InputDataReaderException(String file) {
         super("'" + file + "' contains bad data format");
      }
   }
   
   StreamTokenizer _tokenizer;
   Reader _reader;
   String _fileName;

   public InputDataReader(String fileName) throws IOException {
      _reader = new FileReader(fileName);
      _fileName = fileName;
    
      _tokenizer = new StreamTokenizer(_reader);
    
      // State the '"', '\'' as white spaces.
      _tokenizer.whitespaceChars('"', '"');
      _tokenizer.whitespaceChars('\'', '\'');
    
      // State the '[', ']' as normal characters.
      _tokenizer.ordinaryChar('[');
      _tokenizer.ordinaryChar(']');
      _tokenizer.ordinaryChar(',');
   }

   protected void finalize() throws Throwable {
      _reader.close();
   }

   double readDouble() throws InputDataReaderException,
                              IOException {
      int ntType = _tokenizer.nextToken();
      
      if ( ntType != StreamTokenizer.TT_NUMBER )
         throw new InputDataReaderException(_fileName);
      
      return _tokenizer.nval;
   }
     
   int readInt() throws InputDataReaderException,
                        IOException {
      int ntType = _tokenizer.nextToken();
    
      if ( ntType != StreamTokenizer.TT_NUMBER )
         throw new InputDataReaderException(_fileName);
      
      return (new Double(_tokenizer.nval)).intValue();
   }

   String readString() throws InputDataReaderException,
                              IOException {
      int ntType = _tokenizer.nextToken();
    
      if ( ntType != StreamTokenizer.TT_WORD )
         throw new InputDataReaderException(_fileName);
      
      return _tokenizer.sval;
   }
   
   double[] readDoubleArray() throws InputDataReaderException,
                                     IOException {
      int ntType = _tokenizer.nextToken(); // Read the '['
      
      if ( ntType != '[' )
         throw new InputDataReaderException(_fileName);
      
      Vector values = new Vector();
      ntType = _tokenizer.nextToken();
      while (ntType == StreamTokenizer.TT_NUMBER) {
         values.add(new Double(_tokenizer.nval));
         ntType = _tokenizer.nextToken();
         
         if ( ntType == ',' ) {
            ntType = _tokenizer.nextToken();
         }
         else if ( ntType != ']' ) {
            throw new InputDataReaderException(_fileName);
         }
      }
      
      if ( ntType != ']' )
         throw new InputDataReaderException(_fileName);
    
      // Fill the array.
      double[] res = new double[values.size()];
      for (int i = 0; i < values.size(); i++) {
         res[i] = ((Double)values.elementAt(i)).doubleValue();
      }
      
      return res;
   }

   double[][] readDoubleArrayArray() throws InputDataReaderException,
                                            IOException {
      int ntType = _tokenizer.nextToken(); // Read the '['
      
      if ( ntType != '[' )
         throw new InputDataReaderException(_fileName);
      
      Vector values = new Vector();
      ntType = _tokenizer.nextToken();
      
      while (ntType == '[') {
         _tokenizer.pushBack();
         
         values.add(readDoubleArray());
         
         ntType = _tokenizer.nextToken();
         if      ( ntType == ',' ) {
           ntType = _tokenizer.nextToken();
         }
         else if ( ntType != ']' ) {
           throw new InputDataReaderException(_fileName);
         }
      }
    
      if ( ntType != ']' )
         throw new InputDataReaderException(_fileName);
    
      // Fill the array.
      double[][] res = new double[values.size()][];
      for (int i = 0; i < values.size(); i++) 
         res[i] = (double[])values.elementAt(i);
      
      return res;
   }

   int[] readIntArray() throws InputDataReaderException,
                               IOException {
      int ntType = _tokenizer.nextToken(); // Read the '['
      
      if ( ntType != '[' )
         throw new InputDataReaderException(_fileName);
      
      Vector values = new Vector();
      ntType = _tokenizer.nextToken();
      while (ntType == StreamTokenizer.TT_NUMBER) {
         values.add(new Double(_tokenizer.nval));
         ntType = _tokenizer.nextToken();
         
         if      ( ntType == ',' ) {
            ntType = _tokenizer.nextToken();
         }
         else if ( ntType != ']' ) {
            throw new InputDataReaderException(_fileName);
         }
      }
      
      if ( ntType != ']' )
         throw new InputDataReaderException(_fileName);
    
      // Fill the array.
      int[] res = new int[values.size()];
      for (int i = 0; i < values.size(); i++)
         res[i] = ((Double)values.elementAt(i)).intValue();
      
      return res;
   }

   int[][] readIntArrayArray() throws InputDataReaderException,
                                      IOException {
      int ntType = _tokenizer.nextToken(); // Read the '['
      
      if ( ntType != '[' )
         throw new InputDataReaderException(_fileName);
      
      Vector values = new Vector();
      ntType = _tokenizer.nextToken();
      
      while (ntType == '[') {
         _tokenizer.pushBack();
         
         values.add(readIntArray());
         
         ntType = _tokenizer.nextToken();
         if      ( ntType == ',' ) {
            ntType = _tokenizer.nextToken();
         }
         else if ( ntType != ']' ) {
            throw new InputDataReaderException(_fileName);
         }
      }
    
      if ( ntType != ']' )
         throw new InputDataReaderException(_fileName);
    
      // Fill the array.
      int[][] res = new int[values.size()][];
      for (int i = 0; i < values.size(); i++)
         res[i] = (int[])values.elementAt(i);
      
      return res;
   }

   String[] readStringArray() throws InputDataReaderException,
                                     IOException {
      int ntType = _tokenizer.nextToken(); // Read the '['
      
      if ( ntType != '[' )
         throw new InputDataReaderException(_fileName);
      
      Vector values = new Vector();
      ntType = _tokenizer.nextToken();
      while (ntType == StreamTokenizer.TT_WORD) {
         values.add(_tokenizer.sval);
         ntType = _tokenizer.nextToken();
         
         if      ( ntType == ',' ) {
            ntType = _tokenizer.nextToken();
         }
         else if ( ntType != ']' ) {
            throw new InputDataReaderException(_fileName);
         }
      }
      
      if ( ntType != ']' )
         throw new InputDataReaderException(_fileName);
    
      // Fill the array.
      String[] res = new String[values.size()];
      for (int i = 0; i < values.size(); i++) 
         res[i] = (String)values.elementAt(i);
      
      return res;
   }

   String[][] readStringArrayArray() throws InputDataReaderException,
                                            IOException {
      int ntType = _tokenizer.nextToken(); // Read the '['
      
      if ( ntType != '[' )
         throw new InputDataReaderException(_fileName);
      
      Vector values = new Vector();
      ntType = _tokenizer.nextToken();
      
      while (ntType == '[') {
         _tokenizer.pushBack();
         
         values.add(readStringArray());
         
         ntType = _tokenizer.nextToken();
         if      ( ntType == ',' ) {
            ntType = _tokenizer.nextToken();
         }
         else if ( ntType != ']' ) {
            throw new InputDataReaderException(_fileName);
         }
      }
    
      if ( ntType != ']' )
         throw new InputDataReaderException(_fileName);
    
      // Fill the array.
      String[][] res = new String[values.size()][];
      for (int i = 0; i < values.size(); i++)
         res[i] = (String[])values.elementAt(i);
      
      return res;
   }
}
