// --------------------------------------------------------------------------
// File: examples/src/LPex3.java
// Version 10.1  
// --------------------------------------------------------------------------
//  Copyright (C) 1999-2006 by ILOG.
//  All Rights Reserved.
//  Permission is expressly granted to use this example in the
//  course of developing applications that use ILOG products.
// --------------------------------------------------------------------------
//
// LPex3.java, example of adding constraints to solve a problem
//
// Modified example from Chvatal, "Linear Programming", Chapter 26.
//   minimize  c*x
//   subject to  Hx = d
//               Ax = b
//               l <= x <= u
//   where
//
//   H = ( -1  0  1  0  1  0  0  0 )  d = ( -3 )
//       (  1 -1  0  1  0  0  0  0 )      (  1 )
//       (  0  1 -1  0  0  1 -1  0 )      (  4 )
//       (  0  0  0 -1  0 -1  0  1 )      (  3 )
//       (  0  0  0  0 -1  0  1 -1 )      ( -5 )
//
//   A = (  2  1 -2 -1  2 -1 -2 -3 )  b = (  4 )
//       (  1 -3  2  3 -1  2  1  1 )      ( -2 )
//
//   c = ( -9  1  4  2 -8  2  8 12 ) 
//   l = (  0  0  0  0  0  0  0  0 )
//   u = ( 50 50 50 50 50 50 50 50 )
//
//
//
//  Treat the constraints with A as the complicating constraints, and
//  the constraints with H as the "simple" problem.
//  
//  The idea is to solve the simple problem first, and then add the
//  constraints for the complicating constraints, and solve with dual.

import ilog.concert.*;
import ilog.cplex.*;


public class LPex3 {
  public static void main(String[] args) {
    try {
      int ncols = 8;

      IloCplex cplex = new IloCplex();
      IloLPMatrix lp = cplex.addLPMatrix();

      // add empty corresponding to new variables columns to lp
      IloNumVar[] x = cplex.numVarArray(cplex.columnArray(lp, ncols), 0.0, 50.0);

      // add rows to lp
      double[]   rhs0 = {-3.0, 1.0, 4.0, 3.0, -5.0};
      double[][] val0 = {{-1.0,  1.0,  1.0},
                         { 1.0, -1.0,  1.0},
                         { 1.0, -1.0,  1.0, -1.0},
                         {-1.0, -1.0,  1.0},
                         {-1.0,  1.0,  -1.0}};
      int[][]    ind0 = {{0, 2, 4},
                         {0, 1, 3},
                         {1, 2, 5, 6},
                         {3, 5, 7},
                         {4, 6, 7}};
      lp.addRows(rhs0, rhs0, ind0, val0);

      // add the objective function
      double[] objvals = {-9.0, 1.0, 4.0, 2.0, -8.0, 2.0, 8.0, 12.0};
      cplex.addMinimize(cplex.scalProd(x, objvals));

      // Solve initial problem with the network optimizer
      cplex.setParam(IloCplex.IntParam.RootAlg, IloCplex.Algorithm.Network);
      cplex.solve();
      System.out.println("After network optimization, objective is "
                         + cplex.getObjValue());

      // add rows from matrix A to lp
      double[]   rhs1 = {4.0, -2.0};
      double[][] val1 = {{2.0,  1.0, -2.0, -1.0,  2.0, -1.0, -2.0, -3.0},
                         {1.0, -3.0,  2.0,  3.0, -1.0,  2.0,  1.0,  1.0}};
      int[][]    ind1 = {{0, 1, 2, 3, 4, 5, 6, 7},
                         {0, 1, 2, 3, 4, 5, 6, 7}};
      lp.addRows(rhs1, rhs1, ind1, val1);

      // Because the problem is dual feasible with the rows added, using
      // the dual simplex method is indicated.
      cplex.setParam(IloCplex.IntParam.RootAlg, IloCplex.Algorithm.Dual);
      if ( cplex.solve() ) {
        System.out.println("Solution status = " + cplex.getStatus());
        System.out.println("Solution value  = " + cplex.getObjValue());

        double[] sol = cplex.getValues(lp);
        for (int j = 0; j < ncols; ++j) {
           System.out.println("Column: " + j + " Value = " + sol[j]);
        }
      }
      cplex.end();
    }
    catch (IloException e) {
      System.err.println("Concert exception '" + e + "' caught");
    }
  }
}
