// --------------------------------------------------------------------------
// File: examples/src/Transport.java
// Version 10.1  
// --------------------------------------------------------------------------
//  Copyright (C) 2001-2006 by ILOG.
//  All Rights Reserved.
//  Permission is expressly granted to use this example in the
//  course of developing applications that use ILOG products.
// --------------------------------------------------------------------------

import ilog.cplex.*;
import ilog.concert.*;

public class Transport {
   public static void main(String[] args) {
     if ( args.length < 1 ) {
         System.err.println("Usage: java Transport <type>");
         System.err.println("  type = 0 -> convex  piecewise linear model");
         System.err.println("  type = 1 -> concave piecewise linear model");
         return;
     }

     try {
        IloCplex cplex = new IloCplex();
      
        int nbDemand = 4;
        int nbSupply = 3;
        double[] supply = {1000.0, 850.0, 1250.0};
        double[] demand = {900.0, 1200.0, 600.0, 400.0};
      
        IloNumVar[][] x = new IloNumVar[nbSupply][];
        IloNumVar[][] y = new IloNumVar[nbSupply][];
      
        for (int i = 0; i < nbSupply; i++) {
           x[i] = cplex.numVarArray(nbDemand, 0., Double.MAX_VALUE);
           y[i] = cplex.numVarArray(nbDemand, 0., Double.MAX_VALUE);
        } 
      
        for (int i = 0; i < nbSupply; i++)       // supply must meet demand
           cplex.addEq(cplex.sum(x[i]), supply[i]);
      
        for (int j = 0; j < nbDemand; j++) {     // demand must meet supply
           IloLinearNumExpr v = cplex.linearNumExpr(); 
           for(int i = 0; i < nbSupply; i++)
              v.addTerm(1., x[i][j]);
           cplex.addEq(v, demand[j]);
        }      
      
        double[] points;
        double[] slopes;
        if ( args[0].charAt(0) == '0' ) {         // convex case
           points = new double[] {200.0, 400.0};
           slopes = new double[] { 30.0, 80.0, 130.0};
        }
        else {                                  // concave case
           points = new double[] {200.0, 400.0};
           slopes = new double[] {120.0, 80.0, 50.0};
        }
        for (int i = 0; i < nbSupply; ++i) {
           for (int j = 0; j < nbDemand; ++j) {
              cplex.addEq(y[i][j],
                          cplex.piecewiseLinear(x[i][j],
                                                points, slopes, 0.0, 0.0));
           }
        }
      
        IloLinearNumExpr expr = cplex.linearNumExpr();
        for (int i = 0; i < nbSupply; ++i) {
           for (int j = 0; j < nbDemand; ++j) {
              expr.addTerm(y[i][j], 1.);   
           }
        }
        
        cplex.addMinimize(expr);
      
        if ( cplex.solve() ) {
           System.out.println(" - Solution: "); 
           for (int i = 0; i < nbSupply; ++i) {
              System.out.print("   " + i + ": ");
              for (int j = 0; j < nbDemand; ++j)
                 System.out.print("" + cplex.getValue(x[i][j]) + "\t");
              System.out.println();
           }
           System.out.println("   Cost = " + cplex.getObjValue()); 
        }
        cplex.end();
     }
     catch (IloException exc) {
        System.out.println(exc);
     }
   }
}
