/*------------------------------------------------------------------------*/
/*  File: examples/src/colsteel.c                                         */
/*  Version 10.1                                                          */
/*------------------------------------------------------------------------*/
/*  Copyright (C) 1997-2006 by ILOG.                                      */
/*  All Rights Reserved.                                                  */
/*  Permission is expressly granted to use this example in the            */
/*  course of developing applications that use ILOG products.             */
/*------------------------------------------------------------------------*/

/* This example is an implementation of the model called "steelT.mod"
 * on page 58 in the AMPL book by Fourer, Gay and Kernighan.  In the AMPL
 * example, a multiperiod production model is given, with data
 * for 4 weeks.  
 *
 * The parameters for the model are:
 *   PROD (indexed by p<NUMPROD)  a set of NUMPROD products
 *   TWEEKS = 4 (indexed by t<4)  the number of weeks
 *   rate[p]                      rate of production, product p
 *   inv0[p]                      initial inventory, product p
 *   avail[t]                     hours available in week t
 *   market[p][t]                 limit on demand for product p, week t
 *   prodcost[p]                  production cost per unit of product p 
 *   invcost[p]                   inventory cost per unit of product p
 *   revenue[p][t]                revenue per unit, product p, week t
 *
 * The decision variables of the model are:
 *   Make[p][t]                 amount produced, product p, week t
 *   Inv[p][t]                  amount inventoried, product p, week t
 *   Sell[p][t]                 amount sold, product p, week t
 * 
 * The objective function is to
 * 
 * maximize  sum(over p,t) (
 *                 revenue[p][t] * Sell[p][t]
 *               - prodcost[p]   * Make[p][t]
 *               - invcost[p]    * Inv[p][t]   )
 *
 * The constraints are
 * 
 *  For each t:   (time constraint)
 *      sum(over p)  ( ( 1/rate[p] ) * Make[p][t] ) <= avail[t]
 * 
 *  For each pair (p,t) (t>0): (balance constraint)
 *      Make[p][t] + Inv[p][t-1] - Sell[p][t] - Inv[p][t] = 0
 *
 *  For each p, (t=0): (balance constraint)
 *      Make[p][0] - Sell[p][0] - Inv[p][0] = -inv0[p]
 *
 *  The bounds on the variables are:
 *    All variables are nonnegative ( >= 0 )
 *    For each triple (p,t),
 *       Sell[p][t] <= market[p][t]
 *    All other variables have infinite upper bounds.
 *
 *
 *  To generate the CPLEX data structure for this model, we need to
 *    make an association between each row and an index number,
 *    and each variable and an index number.  
 *  For the rows:
 *    Constraint  time(t)      will be row i = t
 *    Constraint  balance(p,t) will be row 
 *                    i = TWEEKS + p*TWEEKS + t
 *
 *  For the columns:
 *    Make[p][t]  will be column 
 *            j = p*TWEEKS + t
 *    Inv[p][t]   will be column 
 *            j = (NUMPROD+p)*TWEEKS + t
 *    Sell[p][t]  will be column
 *            j = (2*NUMPROD+p)*TWEEKS + t
 *
 *  This part of the code generates the problem by column.
 * 
 *  Note:  In the AMPL book, time was counted from 1 to T, with
 *     0 corresponding to the initial inventory levels.  In our
 *     implementation, we have chosen to have time run from period 0
 *     to period (TWEEKS-1) and place the initial inventory
 *     levels on the right-hand side of the balance constraints for
 *     time period 0.
 */


/* This file contains the code that generates an instance of the 
 * steelt example. The idea is to pass in the data defining the model, 
 * and place the model characteristics into the lp object already
 * created by the caller.  The input parameters are defined as
 *
 *  numprod     number of products
 *  tweeks      number of weeks
 *  rate        array of numprod doubles containing rate of production
 *  inv0        array of numprod doubles containing initial inventories
 *  avail       array of tweeks doubles containing available hours
 *  flatmarket  array of tweeks*numprod doubles containing limit on demand
 *                 for product p, time t in position flatmarket[p*tweeks+t]
 *  prodcost    array of numprod doubles containing production costs
 *  invcost     array of numprod doubles containing inventory costs
 *  flatrevenue array of tweeks*numprod doubles containing revenue for
 *                 product p, time t in position flatrevenue[p*tweeks+t]
 *
 *  The result of the routine is that the problem object lp is modified
 *  to hold the entire model.
 *  
 *  This version of the gensteel function generates the model by creating
 *  the matrix coefficients by column. 
 */

#include <stdio.h>
#include <stdlib.h>

#include <ilcplex/cplex.h>

int 
gensteel (int numprod, int tweeks, double *rate,
          double *inv0, double *avail, double *flatmarket,
          double *prodcost, double *invcost, double *flatrevenue,
          CPXENVptr env, CPXLPptr lp)
{
   double *obj      = NULL;
   char   *sense    = NULL;
   int    *cmatbeg  = NULL;
   int    *cmatind  = NULL;
   double *cmatval  = NULL;
   double *lb       = NULL;
   double *ub       = NULL;

   CPXCHANNELptr  cpxerror = NULL;

   int  t,p,k;      /* Various loop counters */

   int  status = 0;

   status = CPXgetchannels (env, NULL, NULL, &cpxerror, NULL);
   if ( status )  goto TERMINATE;

   /* Set the objective sense to be maximize */

   CPXchgobjsen (env, lp, CPX_MAX);

   /* Set up the constraints for the problem */

   /* First do the time constraints.  Allocate space for a sense array. */

   sense = (char *) malloc (tweeks*sizeof(char));
   if ( sense == NULL ) {
      status = -2;
      goto TERMINATE;
   }

   for (t = 0; t < tweeks; t++) {
      sense[t] = 'L';
   }

   status = CPXnewrows (env, lp, tweeks, avail, sense, NULL, NULL);
   if ( status ) {
      CPXmsg (cpxerror, 
              "CPXnewrows failed to add time constraints.  Error %d\n",
              status);
      goto TERMINATE;
   }

   /* Free up the temporary sense array */
   free (sense);  sense = NULL;

   /* Now do the balance constraints.  
      Can do this without temporary arrays, because the only nonzero
      is the negative of the inventory levels.  */

   for (p = 0; p < numprod; p++) {
      double  temprhs = -inv0[p];

      /* Fill in the initial inventory level */
      status   = CPXnewrows (env, lp, 1, &temprhs, NULL, NULL, NULL);
      if ( status ) {
         CPXmsg (cpxerror, "%sfor product %d.  Error %d.\n", 
                 "CPXnewrows failed to add initial inventory constraint\n",
                 p, status);
         goto TERMINATE;
      }

      /* The remaining balance constraints have 0.0 as the rhs value,
         and are equality constraints.  No need to specify the arrays */

      status   = CPXnewrows (env, lp, tweeks-1, NULL, NULL, NULL, NULL);
      if ( status ) {
         CPXmsg (cpxerror, "%sfor product %d.  Error %d.\n", 
                 "CPXnewrows failed to add other inventory constraints\n",
                 p, status);
         goto TERMINATE;
      }

   }

   /* Now, add the variables.  For each set of variables, we allocate
    * arrays to hold the data for the CPXaddcols() calls, and then free
    * them up, so that each set of variables is maintained in a "local"
    * piece of code.
    */

   /* First, do the Make variables.  Each Make[p][t] variable has
    * objective coefficient -prodcost[p], and bounds of (0, +infinity).
    * Each Make[p][t] variable appears in the constraints 
    * time(t) and balance(p,t).
    * Create temporary arrays to hold the objective coefficients,
    * lower and upper bounds, and matrix coefficients for one product.
    */

   obj     = (double *) malloc (tweeks*sizeof(double));
   cmatbeg = (int    *) malloc (tweeks*sizeof(int));
   cmatind = (int    *) malloc (2*tweeks*sizeof(int));
   cmatval = (double *) malloc (2*tweeks*sizeof(double));

   if ( obj     == NULL ||
        cmatbeg == NULL ||
        cmatind == NULL ||
        cmatval == NULL   ) {
      status = -2;
      goto TERMINATE;
   }

   for (p = 0; p < numprod; p++) {
      k = 0;     /* Reset the nonzero count for each product */
      for (t = 0; t < tweeks; t++) {
         cmatbeg[t] = k;
         obj[t]     = -prodcost[p];
         cmatind[k] = t;
         cmatval[k] = 1.0/rate[p];
         k++;
         cmatind[k] = (p + 1)*tweeks + t;
         cmatval[k] = 1.0;
         k++;
      }
      status = CPXaddcols (env, lp, tweeks, k, obj, cmatbeg, cmatind,
                           cmatval, NULL, NULL, NULL);
      if ( status ) {
         CPXmsg (cpxerror, "%sfor product %d.  Error %d.\n", 
                 "CPXaddcols failed to add Make variables\n",
                 p, status);
         goto TERMINATE;
      }
   }

   /* Free up allocated memory used to add Make variables */

   free (obj);     obj = NULL;
   free (cmatbeg); cmatbeg = NULL;
   free (cmatind); cmatind = NULL;
   free (cmatval); cmatval = NULL;
   
   /* Now do the Inv variables in (p,t) order.  Each Inv[p][t] variable
    * has objective coefficient -invcost[p], and bounds of (0, +infinity).
    * If t is not tweeks-1, then Inv[p][t] appears in constraints
    *   balance(p,t) and balance(p,t+1).
    * If t is tweeks-1, then Inv[p][t] appears only in
    *   constraint balance(p,t).
    * Create temporary arrays to hold the objective coefficients,
    * lower and upper bounds, and matrix coefficients for one product.
    */

   obj     = (double *) malloc (tweeks*sizeof(double));
   cmatbeg = (int    *) malloc (tweeks*sizeof(int));
   cmatind = (int    *) malloc (2*tweeks*sizeof(int));
   cmatval = (double *) malloc (2*tweeks*sizeof(double));

   if ( obj     == NULL ||
        cmatbeg == NULL ||
        cmatind == NULL ||
        cmatval == NULL   ) {
      status = -2;
      goto TERMINATE;
   }

   for (p = 0; p < numprod; p++) {
      k = 0;   /* Reset the nonzero count for each product */
      for (t = 0; t < tweeks; t++) {
         obj[t]     = -invcost[p];
         cmatbeg[t] = k;
         cmatind[k] = (p+1)*tweeks + t;
         cmatval[k] = -1.0;
         k++;
         cmatind[k] = (p+1)*tweeks + t+1;
         cmatval[k] = 1.0;
         k++;
      }
      /* Now repair the coefficient for t=tweeks-1 by passing k-1 as
       * the number of nonzeros, so that the last coefficient is 
       * ignored. 
       */
      status = CPXaddcols (env, lp, tweeks, k-1, obj, cmatbeg, cmatind,
                           cmatval, NULL, NULL, NULL);
      if ( status ) {
         CPXmsg (cpxerror, "%sfor product %d.  Error %d.\n", 
                 "CPXaddcols failed to add Inv variables\n",
                 p, status);
         goto TERMINATE;
      }
   }

   /* Free up allocated memory used to add Inv variables */

   free (obj);     obj = NULL;
   free (cmatbeg); cmatbeg = NULL;
   free (cmatind); cmatind = NULL;
   free (cmatval); cmatval = NULL;
   

   /* Now do the Sell[p][t] variables.  The objective coefficients of
    * Sell[p][t] is flatrevenue[p*tweeks+t], which means that
    * we can pull the objective coefficients from the slice of that
    * array.  The lower bounds are 0.0, and the upper bounds are
    * flatmarket[p*tweeks+t], which means that we can pull the
    * upper bounds from the slice of that array.  Each Sell variable
    * appears only in the balance(p,t) constraint. 
    * Create temporary arrays to hold the bounds, and matrix 
    * coefficients for one product.
    */

   cmatbeg = (int    *) malloc (tweeks*sizeof(int));
   cmatind = (int    *) malloc (tweeks*sizeof(int));
   cmatval = (double *) malloc (tweeks*sizeof(double));

   if ( cmatbeg == NULL ||
        cmatind == NULL ||
        cmatval == NULL   ) {
      status = -2;
      goto TERMINATE;
   }


   for (p = 0; p < numprod; p++) {
      k = 0;     /* Reset the nonzero count for each product */
      for (t = 0; t < tweeks; t++) {
         cmatbeg[t] = k;
         cmatind[k] = (p+1)*tweeks + t;
         cmatval[k] = -1.0;
         k++;
      }
      status = CPXaddcols (env, lp, tweeks, k, &flatrevenue[p*tweeks], 
                           cmatbeg, cmatind, cmatval, NULL, 
                           &flatmarket[p*tweeks], NULL);
      if ( status ) {
         CPXmsg (cpxerror, "%sfor product %d.  Error %d.\n", 
                 "CPXaddcols failed to add Sell variables\n",
                 p, status);
         goto TERMINATE;
      }

   }

   /* Free up allocated memory used to add Sell variables */

   free (lb);      lb  = NULL;
   free (cmatbeg); cmatbeg = NULL;
   free (cmatind); cmatind = NULL;
   free (cmatval); cmatval = NULL;

TERMINATE:

   if ( obj     != NULL )   free ( obj );
   if ( sense   != NULL )   free ( sense );
   if ( cmatbeg != NULL )   free ( cmatbeg );
   if ( cmatind != NULL )   free ( cmatind );
   if ( cmatval != NULL )   free ( cmatval );
   if ( lb      != NULL )   free ( lb );
   if ( ub      != NULL )   free ( ub );

   return (status);

} /* END gensteel */
