/*------------------------------------------------------------------------*/
/*  File: examples/src/rowsteel.c                                         */
/*  Version 10.1                                                          */
/*------------------------------------------------------------------------*/
/*  Copyright (C) 1997-2006 by ILOG.                                      */
/*  All Rights Reserved.                                                  */
/*  Permission is expressly granted to use this example in the            */
/*  course of developing applications that use ILOG products.             */
/*------------------------------------------------------------------------*/

/* This example generates the steel production model (described in gensteel.c)
 * by row, rather than by column.  See the comments in rowsteel.c for
 * a description of the model.
 */


#include <stdio.h>
#include <stdlib.h>

#include <ilcplex/cplex.h>


int 
gensteel (int numprod, int tweeks, double *rate,
          double *inv0, double *avail, double *flatmarket,
          double *prodcost, double *invcost, double *flatrevenue,
          CPXENVptr env, CPXLPptr lp)
{
   double *obj      = NULL;
   double *rhs      = NULL;
   char   *sense    = NULL;

   /* Row representation of the model */
   int    *rmatbeg  = NULL;
   int    *rmatind  = NULL;
   double *rmatval  = NULL;

   CPXCHANNELptr  cpxerror = NULL;

   int  t,p,j,k;      /* Various loop counters */

   int  status = 0;

   status = CPXgetchannels (env, NULL, NULL, &cpxerror, NULL);
   if ( status )  goto TERMINATE;

   /* Set the objective sense to be maximize */

   CPXchgobjsen (env, lp, CPX_MAX);

   /* Set up the variables for the problem.  */

   /* Now fill in the bounds and the objective.  
    * For each variable type, we allocate a vector to hold the 
    * objective coefficients for one product, and multiple time
    * periods.  Note that we allocate and free the vector for
    * each decision variable type to maintain the locality of the code.
    */

   /* First, do the Make variables in (p,t) order.  The bounds are
    * (0, infinity), while the objective for Make[p][t] is -prodcost[p]
    */

   obj = (double *) malloc (tweeks*sizeof(double));
   if ( obj == NULL ) {
      status = -2;
      goto TERMINATE;
   }

   for (p = 0; p < numprod; p++) {
      for (t = 0; t < tweeks; t++) {
         obj[t]    = -prodcost[p];
      }
      status = CPXnewcols (env, lp, tweeks, obj, NULL, NULL, NULL, NULL);
      if ( status ) {
         CPXmsg (cpxerror, "%sfor product %d. Error %d.\n",
                 "CPXnewcols failed to add Make variables\n",
                 p, status);
         goto TERMINATE;
      }
   }

   /* Free up the obj array. */

   free (obj);  obj = NULL;
   
   /* Now do the Inv variables in (p,t) order. The bounds are
    * (0, infinity), while the objective for Inv[p][t] is -invcost[p]
    */

   obj = (double *) malloc (tweeks*sizeof(double));
   if ( obj == NULL ) {
      status = -2;
      goto TERMINATE;
   }

   for (p = 0; p < numprod; p++) {
      for (t = 0; t < tweeks; t++) {
         obj[t]    = -invcost[p];
      }
      status = CPXnewcols (env, lp, tweeks, obj, NULL, NULL, NULL, NULL);
      if ( status ) {
         CPXmsg (cpxerror, "%sfor product %d. Error %d.\n",
                 "CPXnewcols failed to add Inv variables\n",
                 p, status);
         goto TERMINATE;
      }
   }

   /* Free up the obj array. */

   free (obj);  obj = NULL;
   
   /* Now do the Sell[p][t] variables in (p,t) order.  The bounds on
    * Sell[p][t] are (0, flatmarket[p*tweeks+t]), and the objective
    * coefficient is flatrevenue[p*tweeks+t].  Because of this
    * structure, we do not need to allocate a temporary objective
    * coefficient array.  
    */

   for (p = 0; p < numprod; p++) {
      status = CPXnewcols (env, lp, tweeks, &flatrevenue[p*tweeks], 
                           NULL, &flatmarket[p*tweeks], NULL, NULL);
      if ( status ) {
         CPXmsg (cpxerror, "%sfor product %d. Error %d.\n",
                 "CPXnewcols failed to add Make variables\n",
                 p, status);
         goto TERMINATE;
      }
   }

   /* Now generate the constraints.  */

   /* There are tweeks time constraints, each with numprod coefficients.
    * We allocate space for the sense values, and space to hold the matrix.
    * The rhs values are in avail, so there is no need to allocate rhs.
    */

   sense   = (char *)   malloc (tweeks*sizeof(char));
   rmatbeg = (int *)    malloc (tweeks*sizeof(int));
   rmatind = (int *)    malloc (tweeks*numprod*sizeof(int));
   rmatval = (double *) malloc (tweeks*numprod*sizeof(double));

   if ( sense   == NULL ||
        rmatbeg == NULL ||
        rmatind == NULL ||
        rmatval == NULL   ) {
      status = -2;
      goto TERMINATE;
   }

   /* Now generate the time constraints.  The senses are all 'L'.
    * There are numprod  nonzeros in each of these constraints, 
    * with nonzero value (1/rate[p]) for variable Make[p][t]. */

   k = 0;   /* The count of the nonzeros */

   for (t = 0; t < tweeks; t++) {
      sense[t] = 'L';
      rmatbeg[t] = k;
      for (p = 0; p < numprod; p++) {
         rmatind[k] = p*tweeks + t;  /* Formula for Make variable */
         rmatval[k] = 1.0/rate[p];
         k++;
      }
   }

   status = CPXaddrows (env, lp, 0, tweeks, k, avail, sense, 
                        rmatbeg, rmatind, rmatval, NULL, NULL);
   if ( status ) {
      CPXmsg (cpxerror, 
              "CPXaddrows failed to add time constraints. Error %d.\n",
              status);
      goto TERMINATE;
   }

   free (sense);   sense   = NULL;
   free (rmatbeg); rmatbeg = NULL;
   free (rmatind); rmatind = NULL;
   free (rmatval); rmatval = NULL;

   /* Allocate space for each product's balance constraints.  rhs
    * and sense are needed for each time period, and each constraint
    * balance[p,t] has 4 nonzeros. 
    */ 

   rhs     = (double *) malloc (tweeks*sizeof(double));
   rmatbeg = (int *)    malloc (tweeks*sizeof(int));
   rmatind = (int *)    malloc (4*tweeks*sizeof(int));
   rmatval = (double *) malloc (4*tweeks*sizeof(double));

   if ( rhs     == NULL ||
        rmatbeg == NULL ||
        rmatind == NULL ||
        rmatval == NULL   ) {
      status = -2;
      goto TERMINATE;
   }

   /* Now generate the balance constraints.  Add rows by product.
    * The rhs values are -inv0[p] for the first time period,
    * and 0.0 otherwise.  The senses are all 'E'.
    * Handle t=0 specially.
    * Use order of variables Inv[t-1], Make, Inv[t], Sell so that
    * rmatind is in column order.
    */

   for (p = 0; p < numprod; p++) {
   
      k = 0;  /* Initialize the nonzero count for each product */

      for (t = 0; t < tweeks; t++) {
         rmatbeg[t] = k;
         if ( t > 0 ) {
            rhs[t]   = 0.0;
            j = (numprod+p)*tweeks + t-1;  /* Inv[p][t-1] index */
            rmatind[k] = j;
            rmatval[k] = 1.0;
            k++;
         }
         else {
            rhs[t] = -inv0[p];
         }
         rmatind[k] = p*tweeks + t;   /* Make[p][t] index */
         rmatval[k] = 1.0;
         k++;
         rmatind[k] = (numprod+p)*tweeks + t;  /* Inv[p][t] index */
         rmatval[k] = -1.0;
         k++;
         rmatind[k] = (2*numprod+p)*tweeks + t;  /* Sell[p][t] index */
         rmatval[k] = -1.0;
         k++;
      }
      /* Now add the rows to the problem */

      status = CPXaddrows (env, lp, 0, tweeks, k, rhs, NULL, 
                           rmatbeg, rmatind, rmatval, NULL, NULL);
      if ( status ) {
         CPXmsg (cpxerror, "%sfor product %d. Error %d.\n",
                 "CPXaddrows failed to add balance constraint\n",
                 p, status);
         goto TERMINATE;
      }
 
   }

   /* Free up arrays used to build balance constraints */

   free (rhs);     rhs   = NULL;
   free (rmatbeg); rmatbeg = NULL;  
   free (rmatind); rmatind = NULL;  
   free (rmatval); rmatval = NULL;  

TERMINATE:

   if ( rmatbeg != NULL )   free ( rmatbeg );
   if ( rmatind != NULL )   free ( rmatind );
   if ( rmatval != NULL )   free ( rmatval );
   if ( rhs     != NULL )   free ( rhs );
   if ( sense   != NULL )   free ( sense );
   if ( obj     != NULL )   free ( obj );

   return (status);

} /* END gensteel */

