/*------------------------------------------------------------------------*/
/*  File: examples/src/steeldrv.c                                         */
/*  Version 10.1                                                          */
/*------------------------------------------------------------------------*/
/*  Copyright (C) 1997-2006 by ILOG.                                      */
/*  All Rights Reserved.                                                  */
/*  Permission is expressly granted to use this example in the            */
/*  course of developing applications that use ILOG products.             */
/*------------------------------------------------------------------------*/

/* This file contains the driver functions to solve the Steel production
 * model.  The data is defined at compile time, and the routine
 * steelt_opt_and_soln is called to generate and optimize the problem,
 * returning the part of the solution we're interested in.
 */
#include <stdio.h>
#include <stdlib.h>

/* The following is needed to get the CPLEX constants */

#include <ilcplex/cplex.h>


/* Declare external function */

int
   steelt_opt_and_soln ( void (CPXPUBLIC *errmsgfunc)(void *, const char *), 
                         void *handle, int numprod, int tweeks, 
                         double *rate, double *inv0, double *avail, 
                         double *flatmarket, double *prodcost, 
                         double *invcost, double *flatrevenue,
                         int *lpstat_p, double *objval_p, 
                         double *flatmake, double *flatinv, 
                         double *flatsell );

static void CPXPUBLIC
   errmsgfunc (void *handle, const char *string);


#define  TWEEKS   4
#define  NUMPROD  2


/* The routine steelt_opt_and_soln() expects market and revenue to
 * be flat arrays.  By defining them as two dimensional, we can
 * pass pointers to &market[0][0], and &revenue[0][0], and that
 * routine will see them as flat.
 *
 * The same idea applies to the arrays flatmake, flatinv, and flatsell.
 */


int
main()
{

   double rate[NUMPROD] = { 200.0, 140.0 }; 
   double inv0[NUMPROD] = { 10.0, 0.0 };
   double avail[TWEEKS] = { 40.0, 40.0, 32.0, 40.0 };
   double market[NUMPROD][TWEEKS] = 
                          { { 6000.0, 6000.0, 4000.0, 6500.0 },
                            { 4000.0, 2500.0, 3500.0, 4200.0 } };
   double prodcost[NUMPROD] = { 10.0, 11.0 };
   double invcost[NUMPROD]  = {  2.5,  3.0 };
   double revenue[NUMPROD][TWEEKS] = 
                          { { 25.0, 26.0, 27.0, 27.0 },
                            { 30.0, 35.0, 37.0, 39.0 } };
   char   *prodnames[NUMPROD] = { "bands", "coils" };

   double  make[NUMPROD][TWEEKS];
   double  inv[NUMPROD][TWEEKS];
   double  sell[NUMPROD][TWEEKS];
   int     lpstat;
   double  objval;

   int    t,p;      /* Various loop counters */

   int    status;

   status = steelt_opt_and_soln (errmsgfunc, stderr,
                                 NUMPROD, TWEEKS, rate, inv0,
                                 avail, &market[0][0], prodcost,
                                 invcost, &revenue[0][0], &lpstat,
                                 &objval, &make[0][0], &inv[0][0],
                                 &sell[0][0] );
   if ( status )  goto TERMINATE;

   printf ("Solution status:  %d\n", lpstat);

   if ( lpstat != CPX_STAT_OPTIMAL ) {
      printf ("Solution not optimal!!\n");
      goto TERMINATE;
   }

   printf ("Objective value:  %g\n", objval);

   printf ("\nMake:\n%5s ","Week");
   for (p = 0; p < NUMPROD; p++) {
      printf ("%10s ",prodnames[p]);
   }
   printf ("\n");
   for (t = 0; t < TWEEKS; t++) {
      printf ("%5d ",t+1);
      for (p = 0; p < NUMPROD; p++) {
         printf ("%10g ",make[p][t]);
      }
      printf ("\n");
   }

   printf ("\nInv:\n%5s ","Week");
   for (p = 0; p < NUMPROD; p++) {
      printf ("%10s ",prodnames[p]);
   }
   printf ("\n");
   printf ("%5d ",0);
   for (p = 0; p < NUMPROD; p++) {
      printf ("%10g ",inv0[p]);
   }
   printf ("\n");
   for (t = 0; t < TWEEKS; t++) {
      printf ("%5d ",t+1);
      for (p = 0; p < NUMPROD; p++) {
         printf ("%10g ",inv[p][t]);
      }
      printf ("\n");
   }

   printf ("\nSell:\n%5s ","Week");
   for (p = 0; p < NUMPROD; p++) {
      printf ("%10s ",prodnames[p]);
   }
   printf ("\n");
   for (t = 0; t < TWEEKS; t++) {
      printf ("%5d ",t+1);
      for (p = 0; p < NUMPROD; p++) {
         printf ("%10g ",sell[p][t]);
      }
      printf ("\n");
   }


TERMINATE:

   return (status);
} /* END main */


/* This is a very simple message handler to just print the message
 * to the file pointed to by handle.
 */

static void CPXPUBLIC
errmsgfunc (void *handle, const char *string)
{
   FILE *fp;

   fp = (FILE *) handle;  /* Convert handle to a FILE * type */
   fprintf (fp, "%s", string);
} /* END errmsgfunc */
