/*------------------------------------------------------------------------*/
/*  File: examples/src/steelsub.c                                         */
/*  Version 10.1                                                          */
/*------------------------------------------------------------------------*/
/*  Copyright (C) 1997-2006 by ILOG.                                      */
/*  All Rights Reserved.                                                  */
/*  Permission is expressly granted to use this example in the            */
/*  course of developing applications that use ILOG products.             */
/*------------------------------------------------------------------------*/

/* This file contains a function to take as input the parameters defining
 * the steel production model, and produce as output the optimal solution.
 */

/* This function takes as input the following variables:
 *  numprod     number of products
 *  tweeks      number of weeks
 *  rate        array of numprod doubles containing rate of production
 *  inv0        array of numprod doubles containing initial inventories
 *  avail       array of tweeks doubles containing available hours
 *  flatmarket  array of tweeks*numprod doubles containing limit on demand
 *                 for product p, time t in position flatmarket[p*tweeks+t]
 *  prodcost    array of numprod doubles containing production costs
 *  invcost     array of numprod doubles containing inventory costs
 *  flatrevenue array of tweeks*numprod doubles containing revenue for
 *                 product p, time t in position flatrevenue[p*tweeks+t]
 *
 * As output, the function produces the following outputs:
 *  *lpstat_p   receives the status from the LP optimization
 *  *objval_p   receives the optimal objective value
 *  flatmake    should be an array of tweeks*numprod doubles, and will be
 *                 filled with Make[p][t] in position flatmake[p*tweeks+t]
 *  flatinv     should be an array of tweeks*numprod doubles, and will be
 *                 filled with Inv[p][t] in position flatinv[p*tweeks+t]
 *  flatsell    should be an array of tweeks*numprod doubles, and will be
 *                 filled with Sell[p][t] in position flatsell[p*tweeks+t]
 */

#include <ilcplex/cplex.h>
#include <stdlib.h>


int 
   gensteel (int numprod, int tweeks, double *rate,
             double *inv0, double *avail, double *flatmarket,
             double *prodcost, double *invcost, double *flatrevenue,
             CPXENVptr env, CPXLPptr lp);


int
steelt_opt_and_soln (void (CPXPUBLIC *errmsgfunc)(void *, const char *), 
                     void *handle, int numprod, int tweeks, double *rate,
                     double *inv0, double *avail, double *flatmarket,
                     double *prodcost, double *invcost, double *flatrevenue,
                     int *lpstat_p, double *objval_p, double *flatmake, 
                     double *flatinv, double *flatsell)
{
   char   *probname = "steelT";

   CPXENVptr     env = NULL;
   CPXLPptr      lp  = NULL;
   CPXCHANNELptr cpxerror   = NULL;
   CPXCHANNELptr cpxwarning = NULL;
   int    status = 0;
   int    lpstat;
   double objval;


   env = CPXopenCPLEX (&status);

   /* If an error occurs, the status value indicates the reason for
      failure.  We'll call the errmsgfunc with the error, because
      we can't call CPXmsg if CPXopenCPLEX failed.  */

   if ( env == NULL ) {
      char  errmsg[1024];
      if ( errmsgfunc != NULL ) {
         (*errmsgfunc) (handle, "Could not open CPLEX environment.\n");
         CPXgeterrorstring (env, status, errmsg);
         (*errmsgfunc) (handle, errmsg);
      }
      goto TERMINATE;
   }

   status = CPXgetchannels (env, NULL, &cpxwarning, &cpxerror, NULL);
   if ( status ) {
      if ( errmsgfunc != NULL ) {
         (*errmsgfunc) (handle, "Error in CPXgetchannels.\n");
      }
      goto TERMINATE;
   }

   if ( errmsgfunc != NULL ) {
      status = CPXaddfuncdest (env, cpxerror, handle, errmsgfunc);
      if ( !status ) {
	 status = CPXaddfuncdest (env, cpxwarning, handle, errmsgfunc);
      }
   
      if ( status ) {
	 (*errmsgfunc) (handle, "Error in CPXaddfuncdest.\n");
	 goto TERMINATE;
      }
   }

   /* Now that the channels have the error message function, we can
      use CPXmsg for errors. */

   /* Create the problem */

   lp = CPXcreateprob (env, &status, probname);

   if ( lp == NULL ) {
      status = -2;
      CPXmsg (cpxerror, "Failed to create LP.\n");
      goto TERMINATE;
   }

   status = gensteel (numprod, tweeks, rate, inv0, avail, flatmarket,
                      prodcost, invcost, flatrevenue, env, lp);

   if ( status )  goto TERMINATE;


   status = CPXlpopt (env, lp);
   if ( status ) {
      CPXmsg (cpxerror,"Optimization failed. Status %d.\n", status);
      goto TERMINATE;
   }

   status = CPXsolution (env, lp, &lpstat, &objval, NULL, NULL, NULL, NULL);
   if ( status ) {
      CPXmsg (cpxerror, "Solution failed. Status %d.\n", status); 
      goto TERMINATE;
   }

   *objval_p = objval;
   *lpstat_p = lpstat;

   /* Now get the solution.  Use the fact that the variables are
    * added in the order Make, Inv, Sell, and that the slices of
    * the x vector can be used to copy over the solution into
    * the flat... arrays. */

   /* Get the Make variables */
   status = CPXgetx (env, lp, flatmake, 0, numprod*tweeks-1);
   if ( status ) {
      CPXmsg (cpxerror, "CPXgetx failed to get Make solution.  Status %d.\n",
              status);
      goto TERMINATE;
   }

   /* Get the Inv variables */
   status = CPXgetx (env, lp, flatinv, numprod*tweeks, 2*numprod*tweeks-1);
   if ( status ) {
      CPXmsg (cpxerror, "CPXgetx failed to get Inv solution.  Status %d.\n",
              status);
      goto TERMINATE;
   }

   /* Get the Sell variables */
   status = CPXgetx (env, lp, flatsell, 2*numprod*tweeks, 
                     3*numprod*tweeks-1);
   if ( status ) {
      CPXmsg (cpxerror, "CPXgetx failed to get Sell solution.  Status %d.\n",
              status);
      goto TERMINATE;
   }

TERMINATE:
   if ( lp != NULL )   CPXfreeprob (env, &lp);

   if ( errmsgfunc != NULL && env != NULL ) {
      int  dfstat;
      dfstat = CPXdelfuncdest (env, cpxerror, handle, errmsgfunc);
      if ( !dfstat ) {
         dfstat = CPXdelfuncdest (env, cpxwarning, handle, errmsgfunc);
      }
      if ( dfstat ) {
         (*errmsgfunc) (handle, "CPXdelfuncdest failed.\n");
      }
      if ( dfstat && !status )  status = dfstat;
   }
   
   /* Free up the CPLEX environment, if necessary */

   if ( env != NULL ) {
      int  closestat;
      closestat = CPXcloseCPLEX (&env);

      if ( closestat && errmsgfunc != NULL) {
      char  errmsg[1024];
         (*errmsgfunc) (handle, "Could not close CPLEX environment.\n");
         CPXgeterrorstring (env, closestat, errmsg);
         (*errmsgfunc) (handle, errmsg);
      }
      if ( closestat && !status ) status = closestat;
   }

   return (status);
} /* END steelt_opt_and_soln */

